@@ -16,7 +16,6 @@
 """The QoS specs extension"""
 
 from oslo_log import log as logging
-from oslo_utils import strutils
 import six
 import webob
 
@@ -24,9 +23,8 @@ from cinder.api import common
 from cinder.api import extensions
 from cinder.api.openstack import wsgi
 from cinder.api.views import qos_specs as view_qos_specs
-from cinder.api import xmlutil
 from cinder import exception
-from cinder.i18n import _, _LI
+from cinder.i18n import _
 from cinder import rpc
 from cinder import utils
 from cinder.volume import qos_specs
@@ -37,67 +35,9 @@ LOG = logging.getLogger(__name__)
 authorize = extensions.extension_authorizer('volume', 'qos_specs_manage')
 
 
-def make_qos_specs(elem):
-    elem.set('id')
-    elem.set('name')
-    elem.set('consumer')
-    elem.append(SpecsTemplate())
-
-
-def make_associations(elem):
-    elem.set('association_type')
-    elem.set('name')
-    elem.set('id')
-
-
-class SpecsTemplate(xmlutil.TemplateBuilder):
-    def construct(self):
-        return xmlutil.MasterTemplate(xmlutil.make_flat_dict('specs'), 1)
-
-
-class QoSSpecsTemplate(xmlutil.TemplateBuilder):
-    def construct(self):
-        root = xmlutil.TemplateElement('qos_specs')
-        elem = xmlutil.SubTemplateElement(root, 'qos_spec',
-                                          selector='qos_specs')
-        make_qos_specs(elem)
-        return xmlutil.MasterTemplate(root, 1)
-
-
-class QoSSpecsKeyDeserializer(wsgi.XMLDeserializer):
-    def _extract_keys(self, key_node):
-        keys = []
-        for key in key_node.childNodes:
-            key_name = key.tagName
-            keys.append(key_name)
-
-        return keys
-
-    def default(self, string):
-        dom = utils.safe_minidom_parse_string(string)
-        key_node = self.find_first_child_named(dom, 'keys')
-        if not key_node:
-            LOG.info(_LI("Unable to parse XML input."))
-            msg = _("Unable to parse XML request. "
-                    "Please provide XML in correct format.")
-            raise webob.exc.HTTPBadRequest(explanation=msg)
-        return {'body': {'keys': self._extract_keys(key_node)}}
-
-
-class AssociationsTemplate(xmlutil.TemplateBuilder):
-    def construct(self):
-        root = xmlutil.TemplateElement('qos_associations')
-        elem = xmlutil.SubTemplateElement(root, 'associations',
-                                          selector='qos_associations')
-        make_associations(elem)
-        return xmlutil.MasterTemplate(root, 1)
-
-
 def _check_specs(context, specs_id):
-    try:
-        qos_specs.get_qos_specs(context, specs_id)
-    except exception.QoSSpecsNotFound as ex:
-        raise webob.exc.HTTPNotFound(explanation=six.text_type(ex))
+    # Not found exception will be handled at the wsgi level
+    qos_specs.get_qos_specs(context, specs_id)
 
 
 class QoSSpecsController(wsgi.Controller):
@@ -111,7 +51,6 @@ class QoSSpecsController(wsgi.Controller):
                                            method,
                                            payload)
 
-    @wsgi.serializers(xml=QoSSpecsTemplate)
     def index(self, req):
         """Returns the list of qos_specs."""
         context = req.environ['cinder.context']
@@ -132,7 +71,6 @@ class QoSSpecsController(wsgi.Controller):
                                         sort_dirs=sort_dirs)
         return self._view_builder.summary_list(req, specs)
 
-    @wsgi.serializers(xml=QoSSpecsTemplate)
     def create(self, req, body=None):
         context = req.environ['cinder.context']
         authorize(context)
@@ -148,10 +86,14 @@ class QoSSpecsController(wsgi.Controller):
         self.validate_string_length(name, 'name', min_length=1,
                                     max_length=255, remove_whitespaces=True)
         name = name.strip()
+        # Remove name from 'specs' since passing it in as separate param
+        del specs['name']
+
+        # Validate the key-value pairs in the qos spec.
+        utils.validate_dictionary_string_length(specs)
 
         try:
-            qos_specs.create(context, name, specs)
-            spec = qos_specs.get_qos_specs_by_name(context, name)
+            spec = qos_specs.create(context, name, specs)
             notifier_info = dict(name=name, specs=specs)
             rpc.get_notifier('QoSSpecs').info(context,
                                               'qos_specs.create',
@@ -178,7 +120,6 @@ class QoSSpecsController(wsgi.Controller):
 
         return self._view_builder.detail(req, spec)
 
-    @wsgi.serializers(xml=QoSSpecsTemplate)
     def update(self, req, id, body=None):
         context = req.environ['cinder.context']
         authorize(context)
@@ -191,18 +132,13 @@ class QoSSpecsController(wsgi.Controller):
             rpc.get_notifier('QoSSpecs').info(context,
                                               'qos_specs.update',
                                               notifier_info)
-        except exception.QoSSpecsNotFound as err:
+        except (exception.QoSSpecsNotFound, exception.InvalidQoSSpecs) as err:
             notifier_err = dict(id=id, error_message=err)
             self._notify_qos_specs_error(context,
                                          'qos_specs.update',
                                          notifier_err)
-            raise webob.exc.HTTPNotFound(explanation=six.text_type(err))
-        except exception.InvalidQoSSpecs as err:
-            notifier_err = dict(id=id, error_message=err)
-            self._notify_qos_specs_error(context,
-                                         'qos_specs.update',
-                                         notifier_err)
-            raise webob.exc.HTTPBadRequest(explanation=six.text_type(err))
+            # Not found exception will be handled at the wsgi level
+            raise
         except exception.QoSSpecsUpdateFailed as err:
             notifier_err = dict(id=id, error_message=err)
             self._notify_qos_specs_error(context,
@@ -213,16 +149,13 @@ class QoSSpecsController(wsgi.Controller):
 
         return body
 
-    @wsgi.serializers(xml=QoSSpecsTemplate)
     def show(self, req, id):
         """Return a single qos spec item."""
         context = req.environ['cinder.context']
         authorize(context)
 
-        try:
-            spec = qos_specs.get_qos_specs(context, id)
-        except exception.QoSSpecsNotFound as err:
-            raise webob.exc.HTTPNotFound(explanation=six.text_type(err))
+        # Not found exception will be handled at the wsgi level
+        spec = qos_specs.get_qos_specs(context, id)
 
         return self._view_builder.detail(req, spec)
 
@@ -231,10 +164,8 @@ class QoSSpecsController(wsgi.Controller):
         context = req.environ['cinder.context']
         authorize(context)
 
-        force = req.params.get('force', None)
-
         # Convert string to bool type in strict manner
-        force = strutils.bool_from_string(force)
+        force = utils.get_bool_param('force', req.params)
         LOG.debug("Delete qos_spec: %(id)s, force: %(force)s",
                   {'id': id, 'force': force})
 
@@ -249,7 +180,8 @@ class QoSSpecsController(wsgi.Controller):
             self._notify_qos_specs_error(context,
                                          'qos_specs.delete',
                                          notifier_err)
-            raise webob.exc.HTTPNotFound(explanation=six.text_type(err))
+            # Not found exception will be handled at the wsgi level
+            raise
         except exception.QoSSpecsInUse as err:
             notifier_err = dict(id=id, error_message=err)
             self._notify_qos_specs_error(context,
@@ -263,7 +195,6 @@ class QoSSpecsController(wsgi.Controller):
 
         return webob.Response(status_int=202)
 
-    @wsgi.deserializers(xml=QoSSpecsKeyDeserializer)
     def delete_keys(self, req, id, body):
         """Deletes specified keys in qos specs."""
         context = req.environ['cinder.context']
@@ -282,22 +213,16 @@ class QoSSpecsController(wsgi.Controller):
             notifier_info = dict(id=id)
             rpc.get_notifier('QoSSpecs').info(context, 'qos_specs.delete_keys',
                                               notifier_info)
-        except exception.QoSSpecsNotFound as err:
+        except exception.NotFound as err:
             notifier_err = dict(id=id, error_message=err)
             self._notify_qos_specs_error(context,
                                          'qos_specs.delete_keys',
                                          notifier_err)
-            raise webob.exc.HTTPNotFound(explanation=six.text_type(err))
-        except exception.QoSSpecsKeyNotFound as err:
-            notifier_err = dict(id=id, error_message=err)
-            self._notify_qos_specs_error(context,
-                                         'qos_specs.delete_keys',
-                                         notifier_err)
-            raise webob.exc.HTTPBadRequest(explanation=six.text_type(err))
+            # Not found exception will be handled at the wsgi level
+            raise
 
         return webob.Response(status_int=202)
 
-    @wsgi.serializers(xml=AssociationsTemplate)
     def associations(self, req, id):
         """List all associations of given qos specs."""
         context = req.environ['cinder.context']
@@ -316,7 +241,8 @@ class QoSSpecsController(wsgi.Controller):
             self._notify_qos_specs_error(context,
                                          'qos_specs.associations',
                                          notifier_err)
-            raise webob.exc.HTTPNotFound(explanation=six.text_type(err))
+            # Not found exception will be handled at the wsgi level
+            raise
         except exception.CinderException as err:
             notifier_err = dict(id=id, error_message=err)
             self._notify_qos_specs_error(context,
@@ -350,18 +276,13 @@ class QoSSpecsController(wsgi.Controller):
             rpc.get_notifier('QoSSpecs').info(context,
                                               'qos_specs.associate',
                                               notifier_info)
-        except exception.VolumeTypeNotFound as err:
+        except exception.NotFound as err:
             notifier_err = dict(id=id, error_message=err)
             self._notify_qos_specs_error(context,
                                          'qos_specs.associate',
                                          notifier_err)
-            raise webob.exc.HTTPNotFound(explanation=six.text_type(err))
-        except exception.QoSSpecsNotFound as err:
-            notifier_err = dict(id=id, error_message=err)
-            self._notify_qos_specs_error(context,
-                                         'qos_specs.associate',
-                                         notifier_err)
-            raise webob.exc.HTTPNotFound(explanation=six.text_type(err))
+            # Not found exception will be handled at the wsgi level
+            raise
         except exception.InvalidVolumeType as err:
             notifier_err = dict(id=id, error_message=err)
             self._notify_qos_specs_error(context,
@@ -404,18 +325,13 @@ class QoSSpecsController(wsgi.Controller):
             rpc.get_notifier('QoSSpecs').info(context,
                                               'qos_specs.disassociate',
                                               notifier_info)
-        except exception.VolumeTypeNotFound as err:
+        except exception.NotFound as err:
             notifier_err = dict(id=id, error_message=err)
             self._notify_qos_specs_error(context,
                                          'qos_specs.disassociate',
                                          notifier_err)
-            raise webob.exc.HTTPNotFound(explanation=six.text_type(err))
-        except exception.QoSSpecsNotFound as err:
-            notifier_err = dict(id=id, error_message=err)
-            self._notify_qos_specs_error(context,
-                                         'qos_specs.disassociate',
-                                         notifier_err)
-            raise webob.exc.HTTPNotFound(explanation=six.text_type(err))
+            # Not found exception will be handled at the wsgi level
+            raise
         except exception.QoSSpecsDisassociateFailed as err:
             notifier_err = dict(id=id, error_message=err)
             self._notify_qos_specs_error(context,
@@ -444,7 +360,8 @@ class QoSSpecsController(wsgi.Controller):
             self._notify_qos_specs_error(context,
                                          'qos_specs.disassociate_all',
                                          notifier_err)
-            raise webob.exc.HTTPNotFound(explanation=six.text_type(err))
+            # Not found exception will be handled at the wsgi level
+            raise
         except exception.QoSSpecsDisassociateFailed as err:
             notifier_err = dict(id=id, error_message=err)
             self._notify_qos_specs_error(context,
@@ -461,7 +378,6 @@ class Qos_specs_manage(extensions.ExtensionDescriptor):
 
     name = "Qos_specs_manage"
     alias = "qos-specs"
-    namespace = "http://docs.openstack.org/volume/ext/qos-specs/api/v1"
     updated = "2013-08-02T00:00:00+00:00"
 
     def get_resources(self):
