@@ -18,13 +18,10 @@ import functools
 import inspect
 import math
 import time
-from xml.dom import minidom
-from xml.parsers import expat
 
-from lxml import etree
 from oslo_log import log as logging
-from oslo_log import versionutils
 from oslo_serialization import jsonutils
+from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import strutils
 import six
@@ -36,30 +33,21 @@ from cinder.api.openstack import versioned_method
 from cinder import exception
 from cinder import i18n
 from cinder.i18n import _, _LE, _LI
+from cinder import policy
 from cinder import utils
 from cinder.wsgi import common as wsgi
 
 
-XML_NS_V1 = 'http://docs.openstack.org/api/openstack-block-storage/1.0/content'
-XML_NS_V2 = 'http://docs.openstack.org/api/openstack-block-storage/2.0/content'
-XML_NS_ATOM = 'http://www.w3.org/2005/Atom'
-XML_WARNING = False
-
 LOG = logging.getLogger(__name__)
 
 SUPPORTED_CONTENT_TYPES = (
     'application/json',
     'application/vnd.openstack.volume+json',
-    'application/xml',
-    'application/vnd.openstack.volume+xml',
 )
 
 _MEDIA_TYPE_MAP = {
     'application/vnd.openstack.volume+json': 'json',
     'application/json': 'json',
-    'application/vnd.openstack.volume+xml': 'xml',
-    'application/xml': 'xml',
-    'application/atom+xml': 'atom',
 }
 
 
@@ -371,95 +359,6 @@ class JSONDeserializer(TextDeserializer):
         return {'body': self._from_json(datastring)}
 
 
-class XMLDeserializer(TextDeserializer):
-
-    def __init__(self, metadata=None):
-        """Initialize XMLDeserializer.
-
-        :param metadata: information needed to deserialize xml into
-                         a dictionary.
-        """
-        super(XMLDeserializer, self).__init__()
-        self.metadata = metadata or {}
-
-    def _from_xml(self, datastring):
-        plurals = set(self.metadata.get('plurals', {}))
-
-        try:
-            node = utils.safe_minidom_parse_string(datastring).childNodes[0]
-            return {node.nodeName: self._from_xml_node(node, plurals)}
-        except expat.ExpatError:
-            msg = _("cannot understand XML")
-            raise exception.MalformedRequestBody(reason=msg)
-
-    def _from_xml_node(self, node, listnames):
-        """Convert a minidom node to a simple Python type.
-
-        :param listnames: list of XML node names whose subnodes should
-                          be considered list items.
-        """
-        if len(node.childNodes) == 1 and node.childNodes[0].nodeType == 3:
-            return node.childNodes[0].nodeValue
-        elif node.nodeName in listnames:
-            return [self._from_xml_node(n, listnames) for n in node.childNodes]
-        else:
-            result = dict()
-            for attr in node.attributes.keys():
-                result[attr] = node.attributes[attr].nodeValue
-            for child in node.childNodes:
-                if child.nodeType != node.TEXT_NODE:
-                    result[child.nodeName] = self._from_xml_node(child,
-                                                                 listnames)
-            return result
-
-    def find_first_child_named_in_namespace(self, parent, namespace, name):
-        """Search a nodes children for the first child with a given name."""
-        for node in parent.childNodes:
-            if (node.localName == name and
-                    node.namespaceURI and
-                    node.namespaceURI == namespace):
-                return node
-        return None
-
-    def find_first_child_named(self, parent, name):
-        """Search a nodes children for the first child with a given name."""
-        for node in parent.childNodes:
-            if node.nodeName == name:
-                return node
-        return None
-
-    def find_children_named(self, parent, name):
-        """Return all of a nodes children who have the given name."""
-        for node in parent.childNodes:
-            if node.nodeName == name:
-                yield node
-
-    def extract_text(self, node):
-        """Get the text field contained by the given node."""
-        text = []
-        # Cannot assume entire text will be in a single child node because SAX
-        # parsers may split contiguous character data into multiple chunks
-        for child in node.childNodes:
-            if child.nodeType == child.TEXT_NODE:
-                text.append(child.nodeValue)
-        return ''.join(text)
-
-    def default(self, datastring):
-        return {'body': self._from_xml(datastring)}
-
-
-class MetadataXMLDeserializer(XMLDeserializer):
-
-    def extract_metadata(self, metadata_node):
-        """Marshal the metadata attribute of a parsed request."""
-        metadata = {}
-        if metadata_node is not None:
-            for meta_node in self.find_children_named(metadata_node, "meta"):
-                key = meta_node.getAttribute("key")
-                metadata[key] = self.extract_text(meta_node)
-        return metadata
-
-
 class DictSerializer(ActionDispatcher):
     """Default request body serialization."""
 
@@ -477,111 +376,6 @@ class JSONDictSerializer(DictSerializer):
         return jsonutils.dump_as_bytes(data)
 
 
-class XMLDictSerializer(DictSerializer):
-
-    def __init__(self, metadata=None, xmlns=None):
-        """Initialize XMLDictSerializer.
-
-        :param metadata: information needed to deserialize xml into
-                         a dictionary.
-        :param xmlns: XML namespace to include with serialized xml
-        """
-        super(XMLDictSerializer, self).__init__()
-        self.metadata = metadata or {}
-        self.xmlns = xmlns
-
-    def default(self, data):
-        # We expect data to contain a single key which is the XML root.
-        root_key = list(data.keys())[0]
-        doc = minidom.Document()
-        node = self._to_xml_node(doc, self.metadata, root_key, data[root_key])
-
-        return self.to_xml_string(node)
-
-    def to_xml_string(self, node, has_atom=False):
-        self._add_xmlns(node, has_atom)
-        return node.toxml('UTF-8')
-
-    # NOTE (ameade): the has_atom should be removed after all of the
-    # xml serializers and view builders have been updated to the current
-    # spec that required all responses include the xmlns:atom, the has_atom
-    # flag is to prevent current tests from breaking
-    def _add_xmlns(self, node, has_atom=False):
-        if self.xmlns is not None:
-            node.setAttribute('xmlns', self.xmlns)
-        if has_atom:
-            node.setAttribute('xmlns:atom', "http://www.w3.org/2005/Atom")
-
-    def _to_xml_node(self, doc, metadata, nodename, data):
-        """Recursive method to convert data members to XML nodes."""
-        result = doc.createElement(nodename)
-
-        # Set the xml namespace if one is specified
-        # TODO(justinsb): We could also use prefixes on the keys
-        xmlns = metadata.get('xmlns', None)
-        if xmlns:
-            result.setAttribute('xmlns', xmlns)
-
-        # TODO(bcwaldon): accomplish this without a type-check
-        if isinstance(data, list):
-            collections = metadata.get('list_collections', {})
-            if nodename in collections:
-                metadata = collections[nodename]
-                for item in data:
-                    node = doc.createElement(metadata['item_name'])
-                    node.setAttribute(metadata['item_key'], str(item))
-                    result.appendChild(node)
-                return result
-            singular = metadata.get('plurals', {}).get(nodename, None)
-            if singular is None:
-                if nodename.endswith('s'):
-                    singular = nodename[:-1]
-                else:
-                    singular = 'item'
-            for item in data:
-                node = self._to_xml_node(doc, metadata, singular, item)
-                result.appendChild(node)
-        # TODO(bcwaldon): accomplish this without a type-check
-        elif isinstance(data, dict):
-            collections = metadata.get('dict_collections', {})
-            if nodename in collections:
-                metadata = collections[nodename]
-                for k, v in sorted(data.items()):
-                    node = doc.createElement(metadata['item_name'])
-                    node.setAttribute(metadata['item_key'], str(k))
-                    text = doc.createTextNode(str(v))
-                    node.appendChild(text)
-                    result.appendChild(node)
-                return result
-            attrs = metadata.get('attributes', {}).get(nodename, {})
-            for k, v in sorted(data.items()):
-                if k in attrs:
-                    result.setAttribute(k, str(v))
-                else:
-                    node = self._to_xml_node(doc, metadata, k, v)
-                    result.appendChild(node)
-        else:
-            # Type is atom
-            node = doc.createTextNode(str(data))
-            result.appendChild(node)
-        return result
-
-    def _create_link_nodes(self, xml_doc, links):
-        link_nodes = []
-        for link in links:
-            link_node = xml_doc.createElement('atom:link')
-            link_node.setAttribute('rel', link['rel'])
-            link_node.setAttribute('href', link['href'])
-            if 'type' in link:
-                link_node.setAttribute('type', link['type'])
-            link_nodes.append(link_node)
-        return link_nodes
-
-    def _to_xml(self, root):
-        """Convert the xml object to an xml string."""
-        return etree.tostring(root, encoding='UTF-8', xml_declaration=True)
-
-
 def serializers(**serializers):
     """Attaches serializers to a method.
 
@@ -783,15 +577,6 @@ def action_peek_json(body):
     return list(decoded.keys())[0]
 
 
-def action_peek_xml(body):
-    """Determine action to invoke."""
-
-    dom = utils.safe_minidom_parse_string(body)
-    action_node = dom.childNodes[0]
-
-    return action_node.tagName
-
-
 class ResourceExceptionHandler(object):
     """Context manager to handle Resource exceptions.
 
@@ -812,7 +597,7 @@ class ResourceExceptionHandler(object):
             raise Fault(webob.exc.HTTPForbidden(explanation=msg))
         elif isinstance(ex_value, exception.VersionNotFoundForAPIMethod):
             raise
-        elif isinstance(ex_value, exception.Invalid):
+        elif isinstance(ex_value, (exception.Invalid, exception.NotFound)):
             raise Fault(exception.ConvertedException(
                 code=ex_value.code, explanation=six.text_type(ex_value)))
         elif isinstance(ex_value, TypeError):
@@ -858,16 +643,13 @@ class Resource(wsgi.Application):
 
         self.controller = controller
 
-        default_deserializers = dict(xml=XMLDeserializer,
-                                     json=JSONDeserializer)
+        default_deserializers = dict(json=JSONDeserializer)
         default_deserializers.update(deserializers)
 
         self.default_deserializers = default_deserializers
-        self.default_serializers = dict(xml=XMLDictSerializer,
-                                        json=JSONDictSerializer)
+        self.default_serializers = dict(json=JSONDictSerializer)
 
-        self.action_peek = dict(xml=action_peek_xml,
-                                json=action_peek_json)
+        self.action_peek = dict(json=action_peek_json)
         self.action_peek.update(action_peek or {})
 
         # Copy over the actions dictionary
@@ -1060,10 +842,6 @@ class Resource(wsgi.Application):
         return self._process_stack(request, action, action_args,
                                    content_type, body, accept)
 
-    def _is_legacy_endpoint(self, request):
-        version_str = request.api_version_request.get_string()
-        return '1.0' in version_str or '2.0' in version_str
-
     def _process_stack(self, request, action, action_args,
                        content_type, body, accept):
         """Implement the processing stack."""
@@ -1082,9 +860,10 @@ class Resource(wsgi.Application):
             return Fault(webob.exc.HTTPBadRequest(explanation=msg))
 
         if body:
+            decoded_body = encodeutils.safe_decode(body, errors='ignore')
             msg = ("Action: '%(action)s', calling method: %(meth)s, body: "
                    "%(body)s") % {'action': action,
-                                  'body': six.text_type(body),
+                                  'body': six.text_type(decoded_body),
                                   'meth': six.text_type(meth)}
             LOG.debug(strutils.mask_password(msg))
         else:
@@ -1165,15 +944,12 @@ class Resource(wsgi.Application):
         if hasattr(response, 'headers'):
             for hdr, val in response.headers.items():
                 # Headers must be utf-8 strings
-                try:
-                    # python 2.x
-                    response.headers[hdr] = val.encode('utf-8')
-                except Exception:
-                    # python 3.x
-                    response.headers[hdr] = six.text_type(val)
+                val = utils.convert_str(val)
+
+                response.headers[hdr] = val
 
             if (not request.api_version_request.is_null() and
-               not self._is_legacy_endpoint(request)):
+               not _is_legacy_endpoint(request)):
                 response.headers[API_VERSION_REQUEST_HEADER] = (
                     VOLUME_SERVICE + ' ' +
                     request.api_version_request.get_string())
@@ -1283,21 +1059,32 @@ class ControllerMetaclass(type):
         # Find all actions
         actions = {}
         extensions = []
-        versioned_methods = None
+        # NOTE(geguileo): We'll keep a list of versioned methods that have been
+        # added by the new metaclass (dictionary in attribute VER_METHOD_ATTR
+        # on Controller class) and all the versioned methods from the different
+        # base classes so we can consolidate them.
+        versioned_methods = []
+
+        # NOTE(cyeoh): This resets the VER_METHOD_ATTR attribute
+        # between API controller class creations. This allows us
+        # to use a class decorator on the API methods that doesn't
+        # require naming explicitly what method is being versioned as
+        # it can be implicit based on the method decorated. It is a bit
+        # ugly.
+        if bases != (object,) and VER_METHOD_ATTR in vars(Controller):
+            # Get the versioned methods that this metaclass creation has added
+            # to the Controller class
+            versioned_methods.append(getattr(Controller, VER_METHOD_ATTR))
+            # Remove them so next metaclass has a clean start
+            delattr(Controller, VER_METHOD_ATTR)
+
         # start with wsgi actions from base classes
         for base in bases:
             actions.update(getattr(base, 'wsgi_actions', {}))
 
-            if base.__name__ == "Controller":
-                # NOTE(cyeoh): This resets the VER_METHOD_ATTR attribute
-                # between API controller class creations. This allows us
-                # to use a class decorator on the API methods that doesn't
-                # require naming explicitly what method is being versioned as
-                # it can be implicit based on the method decorated. It is a bit
-                # ugly.
-                if VER_METHOD_ATTR in base.__dict__:
-                    versioned_methods = getattr(base, VER_METHOD_ATTR)
-                    delattr(base, VER_METHOD_ATTR)
+            # Get the versioned methods that this base has
+            if VER_METHOD_ATTR in vars(base):
+                versioned_methods.append(getattr(base, VER_METHOD_ATTR))
 
         for key, value in cls_dict.items():
             if not callable(value):
@@ -1311,11 +1098,24 @@ class ControllerMetaclass(type):
         cls_dict['wsgi_actions'] = actions
         cls_dict['wsgi_extensions'] = extensions
         if versioned_methods:
-            cls_dict[VER_METHOD_ATTR] = versioned_methods
+            cls_dict[VER_METHOD_ATTR] = mcs.consolidate_vers(versioned_methods)
 
         return super(ControllerMetaclass, mcs).__new__(mcs, name, bases,
                                                        cls_dict)
 
+    @staticmethod
+    def consolidate_vers(versioned_methods):
+        """Consolidates a list of versioned methods dictionaries."""
+        if not versioned_methods:
+            return {}
+        result = versioned_methods.pop(0)
+        for base_methods in versioned_methods:
+            for name, methods in base_methods.items():
+                method_list = result.setdefault(name, [])
+                method_list.extend(methods)
+                method_list.sort(reverse=True)
+        return result
+
 
 @six.add_metaclass(ControllerMetaclass)
 class Controller(object):
@@ -1421,6 +1221,12 @@ class Controller(object):
             # ranges of valid versions as that is ambiguous
             func_list.sort(reverse=True)
 
+            # NOTE(geguileo): To avoid PEP8 errors when defining multiple
+            # microversions of the same method in the same class we add the
+            # api_version decorator to the function so it can be used instead,
+            # thus preventing method redefinition errors.
+            f.api_version = cls.api_version
+
             return f
 
         return decorator
@@ -1496,6 +1302,23 @@ class Controller(object):
         except exception.InvalidInput as error:
             raise webob.exc.HTTPBadRequest(explanation=error.msg)
 
+    @staticmethod
+    def get_policy_checker(prefix):
+        @staticmethod
+        def policy_checker(req, action, resource=None):
+            ctxt = req.environ['cinder.context']
+            target = {
+                'project_id': ctxt.project_id,
+                'user_id': ctxt.user_id,
+            }
+            if resource:
+                target.update(resource)
+
+            _action = '%s:%s' % (prefix, action)
+            policy.enforce(ctxt, _action, target)
+            return ctxt
+        return policy_checker
+
 
 class Fault(webob.exc.HTTPException):
     """Wrap webob.exc.HTTPException to provide API friendly response."""
@@ -1533,30 +1356,17 @@ class Fault(webob.exc.HTTPException):
             if retry:
                 fault_data[fault_name]['retryAfter'] = retry
 
-        if not req.api_version_request.is_null():
+        if (not req.api_version_request.is_null() and not
+           _is_legacy_endpoint(req)):
             self.wrapped_exc.headers[API_VERSION_REQUEST_HEADER] = (
-                req.api_version_request.get_string())
+                VOLUME_SERVICE + ' ' + req.api_version_request.get_string())
             self.wrapped_exc.headers['Vary'] = API_VERSION_REQUEST_HEADER
 
-        # 'code' is an attribute on the fault tag itself
-        metadata = {'attributes': {fault_name: 'code'}}
-
-        xml_serializer = XMLDictSerializer(metadata, XML_NS_V2)
-
         content_type = req.best_match_content_type()
         serializer = {
-            'application/xml': xml_serializer,
             'application/json': JSONDictSerializer(),
         }[content_type]
 
-        if content_type == 'application/xml':
-            global XML_WARNING
-            if not XML_WARNING:
-                msg = _('XML support has been deprecated and will be removed '
-                        'in the N release.')
-                versionutils.report_deprecated_feature(LOG, msg)
-            XML_WARNING = True
-
         body = serializer.serialize(fault_data)
         if isinstance(body, six.text_type):
             body = body.encode('utf-8')
@@ -1576,6 +1386,11 @@ def _set_request_id_header(req, headers):
         headers['x-compute-request-id'] = context.request_id
 
 
+def _is_legacy_endpoint(request):
+    version_str = request.api_version_request.get_string()
+    return '1.0' in version_str or '2.0' in version_str
+
+
 class OverLimitFault(webob.exc.HTTPException):
     """Rate-limited request response."""
 
@@ -1602,7 +1417,6 @@ class OverLimitFault(webob.exc.HTTPException):
     def __call__(self, request):
         """Serializes the wrapped exception conforming to our error format."""
         content_type = request.best_match_content_type()
-        metadata = {"attributes": {"overLimitFault": "code"}}
 
         def translate(msg):
             locale = request.best_match_language()
@@ -1613,9 +1427,7 @@ class OverLimitFault(webob.exc.HTTPException):
         self.content['overLimitFault']['details'] = \
             translate(self.content['overLimitFault']['details'])
 
-        xml_serializer = XMLDictSerializer(metadata, XML_NS_V2)
         serializer = {
-            'application/xml': xml_serializer,
             'application/json': JSONDictSerializer(),
         }[content_type]
 
