@@ -23,6 +23,7 @@ from oslo_log import log as logging
 from cinder.backup.drivers import posix
 from cinder import exception
 from cinder.i18n import _
+from cinder import interface
 from cinder import utils
 
 LOG = logging.getLogger(__name__)
@@ -44,6 +45,7 @@ CONF = cfg.CONF
 CONF.register_opts(nfsbackup_service_opts)
 
 
+@interface.backupdriver
 class NFSBackupDriver(posix.PosixBackupDriver):
     """Provides backup, restore and delete using NFS supplied repository."""
 
