@@ -25,6 +25,8 @@ from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import timeutils
 
+from cinder.common import constants
+from cinder import db
 from cinder.db import base
 from cinder import exception
 from cinder.i18n import _, _LE, _LW
@@ -172,32 +174,6 @@ class API(base.Base):
     def create_from_src(self, context, name, description=None,
                         cgsnapshot_id=None, source_cgid=None):
         check_policy(context, 'create')
-        cgsnapshot = None
-        orig_cg = None
-        if cgsnapshot_id:
-            try:
-                cgsnapshot = objects.CGSnapshot.get_by_id(context,
-                                                          cgsnapshot_id)
-            except exception.CgSnapshotNotFound:
-                with excutils.save_and_reraise_exception():
-                    LOG.error(_LE("CG snapshot %(cgsnap)s not found when "
-                                  "creating consistency group %(cg)s from "
-                                  "source."),
-                              {'cg': name, 'cgsnap': cgsnapshot_id})
-            else:
-                orig_cg = cgsnapshot.consistencygroup
-
-        source_cg = None
-        if source_cgid:
-            try:
-                source_cg = objects.ConsistencyGroup.get_by_id(context,
-                                                               source_cgid)
-            except exception.ConsistencyGroupNotFound:
-                with excutils.save_and_reraise_exception():
-                    LOG.error(_LE("Source CG %(source_cg)s not found when "
-                                  "creating consistency group %(cg)s from "
-                                  "source."),
-                              {'cg': name, 'source_cg': source_cgid})
 
         kwargs = {
             'user_id': context.user_id,
@@ -209,20 +185,21 @@ class API(base.Base):
             'source_cgid': source_cgid,
         }
 
-        if orig_cg:
-            kwargs['volume_type_id'] = orig_cg.volume_type_id
-            kwargs['availability_zone'] = orig_cg.availability_zone
-            kwargs['host'] = orig_cg.host
-
-        if source_cg:
-            kwargs['volume_type_id'] = source_cg.volume_type_id
-            kwargs['availability_zone'] = source_cg.availability_zone
-            kwargs['host'] = source_cg.host
-
         group = None
         try:
             group = objects.ConsistencyGroup(context=context, **kwargs)
-            group.create()
+            group.create(cg_snap_id=cgsnapshot_id, cg_id=source_cgid)
+        except exception.ConsistencyGroupNotFound:
+            with excutils.save_and_reraise_exception():
+                LOG.error(_LE("Source CG %(source_cg)s not found when "
+                              "creating consistency group %(cg)s from "
+                              "source."),
+                          {'cg': name, 'source_cg': source_cgid})
+        except exception.CgSnapshotNotFound:
+            with excutils.save_and_reraise_exception():
+                LOG.error(_LE("CG snapshot %(cgsnap)s not found when creating "
+                              "consistency group %(cg)s from source."),
+                          {'cg': name, 'cgsnap': cgsnapshot_id})
         except Exception:
             with excutils.save_and_reraise_exception():
                 LOG.error(_LE("Error occurred when creating consistency group"
@@ -237,15 +214,16 @@ class API(base.Base):
             LOG.error(msg)
             raise exception.InvalidConsistencyGroup(reason=msg)
 
-        if cgsnapshot:
-            self._create_cg_from_cgsnapshot(context, group, cgsnapshot)
-        elif source_cg:
-            self._create_cg_from_source_cg(context, group, source_cg)
+        if cgsnapshot_id:
+            self._create_cg_from_cgsnapshot(context, group, cgsnapshot_id)
+        elif source_cgid:
+            self._create_cg_from_source_cg(context, group, source_cgid)
 
         return group
 
-    def _create_cg_from_cgsnapshot(self, context, group, cgsnapshot):
+    def _create_cg_from_cgsnapshot(self, context, group, cgsnapshot_id):
         try:
+            cgsnapshot = objects.CGSnapshot.get_by_id(context, cgsnapshot_id)
             snapshots = objects.SnapshotList.get_all_for_cgsnapshot(
                 context, cgsnapshot.id)
 
@@ -305,8 +283,10 @@ class API(base.Base):
         self.volume_rpcapi.create_consistencygroup_from_src(
             context, group, cgsnapshot)
 
-    def _create_cg_from_source_cg(self, context, group, source_cg):
+    def _create_cg_from_source_cg(self, context, group, source_cgid):
         try:
+            source_cg = objects.ConsistencyGroup.get_by_id(context,
+                                                           source_cgid)
             source_vols = self.db.volume_get_all_by_group(context,
                                                           source_cg.id)
 
@@ -416,7 +396,7 @@ class API(base.Base):
         # to select the target host for this group.
         self.scheduler_rpcapi.create_consistencygroup(
             context,
-            CONF.volume_topic,
+            constants.VOLUME_TOPIC,
             group,
             request_spec_list=request_spec_list,
             filter_properties_list=filter_properties_list)
@@ -448,59 +428,52 @@ class API(base.Base):
 
             return
 
-        if not force and group.status not in (
-                [c_fields.ConsistencyGroupStatus.AVAILABLE,
-                 c_fields.ConsistencyGroupStatus.ERROR]):
-            msg = _("Consistency group status must be available or error, "
-                    "but current status is: %s") % group.status
+        if force:
+            expected = {}
+        else:
+            expected = {'status': (c_fields.ConsistencyGroupStatus.AVAILABLE,
+                                   c_fields.ConsistencyGroupStatus.ERROR)}
+        filters = [~db.cg_has_cgsnapshot_filter(),
+                   ~db.cg_has_volumes_filter(attached_or_with_snapshots=force),
+                   ~db.cg_creating_from_src(cg_id=group.id)]
+        values = {'status': c_fields.ConsistencyGroupStatus.DELETING}
+
+        if not group.conditional_update(values, expected, filters):
+            if force:
+                reason = _('Consistency group must not have attached volumes, '
+                           'volumes with snapshots, or dependent cgsnapshots')
+            else:
+                reason = _('Consistency group status must be available or '
+                           'error and must not have volumes or dependent '
+                           'cgsnapshots')
+            msg = (_('Cannot delete consistency group %(id)s. %(reason)s, and '
+                     'it cannot be the source for an ongoing CG or CG '
+                     'Snapshot creation.')
+                   % {'id': group.id, 'reason': reason})
             raise exception.InvalidConsistencyGroup(reason=msg)
-
-        cgsnapshots = objects.CGSnapshotList.get_all_by_group(
-            context.elevated(), group.id)
-        if cgsnapshots:
-            msg = _("Consistency group %s still has dependent "
-                    "cgsnapshots.") % group.id
-            LOG.error(msg)
-            raise exception.InvalidConsistencyGroup(reason=msg)
-
-        volumes = self.db.volume_get_all_by_group(context.elevated(),
-                                                  group.id)
-
-        if volumes and not force:
-            msg = _("Consistency group %s still contains volumes. "
-                    "The force flag is required to delete it.") % group.id
-            LOG.error(msg)
-            raise exception.InvalidConsistencyGroup(reason=msg)
-
-        for volume in volumes:
-            if volume['attach_status'] == "attached":
-                msg = _("Volume in consistency group %s is attached. "
-                        "Need to detach first.") % group.id
-                LOG.error(msg)
-                raise exception.InvalidConsistencyGroup(reason=msg)
-
-            snapshots = objects.SnapshotList.get_all_for_volume(context,
-                                                                volume['id'])
-            if snapshots:
-                msg = _("Volume in consistency group still has "
-                        "dependent snapshots.")
-                LOG.error(msg)
-                raise exception.InvalidConsistencyGroup(reason=msg)
-
-        group.status = c_fields.ConsistencyGroupStatus.DELETING
-        group.terminated_at = timeutils.utcnow()
-        group.save()
-
         self.volume_rpcapi.delete_consistencygroup(context, group)
 
-    def update(self, context, group, name, description,
-               add_volumes, remove_volumes):
-        """Update consistency group."""
-        if group.status != c_fields.ConsistencyGroupStatus.AVAILABLE:
-            msg = _("Consistency group status must be available, "
-                    "but current status is: %s.") % group.status
-            raise exception.InvalidConsistencyGroup(reason=msg)
+    def _check_update(self, group, name, description, add_volumes,
+                      remove_volumes, allow_empty=False):
+        if allow_empty:
+            if (name is None and description is None
+                    and not add_volumes and not remove_volumes):
+                msg = (_("Cannot update consistency group %(group_id)s "
+                         "because no valid name, description, add_volumes, "
+                         "or remove_volumes were provided.") %
+                       {'group_id': group.id})
+                raise exception.InvalidConsistencyGroup(reason=msg)
+        else:
+            if not (name or description or add_volumes or remove_volumes):
+                msg = (_("Cannot update consistency group %(group_id)s "
+                         "because no valid name, description, add_volumes, "
+                         "or remove_volumes were provided.") %
+                       {'group_id': group.id})
+                raise exception.InvalidConsistencyGroup(reason=msg)
 
+    def update(self, context, group, name, description,
+               add_volumes, remove_volumes, allow_empty=False):
+        """Update consistency group."""
         add_volumes_list = []
         remove_volumes_list = []
         if add_volumes:
@@ -519,50 +492,67 @@ class API(base.Base):
                     "list.") % invalid_uuids
             raise exception.InvalidVolume(reason=msg)
 
-        volumes = self.db.volume_get_all_by_group(context, group.id)
-
         # Validate name.
-        if not name or name == group.name:
+        if name == group.name:
             name = None
 
         # Validate description.
-        if not description or description == group.description:
+        if description == group.description:
             description = None
-
-        # Validate volumes in add_volumes and remove_volumes.
-        add_volumes_new = ""
-        remove_volumes_new = ""
-        if add_volumes_list:
-            add_volumes_new = self._validate_add_volumes(
-                context, volumes, add_volumes_list, group)
-        if remove_volumes_list:
-            remove_volumes_new = self._validate_remove_volumes(
-                volumes, remove_volumes_list, group)
-
-        if (not name and not description and not add_volumes_new and
-                not remove_volumes_new):
-            msg = (_("Cannot update consistency group %(group_id)s "
-                     "because no valid name, description, add_volumes, "
-                     "or remove_volumes were provided.") %
-                   {'group_id': group.id})
-            raise exception.InvalidConsistencyGroup(reason=msg)
+        self._check_update(group, name, description, add_volumes,
+                           remove_volumes, allow_empty)
 
         fields = {'updated_at': timeutils.utcnow()}
 
         # Update name and description in db now. No need to
         # to send them over through an RPC call.
-        if name:
-            fields['name'] = name
-        if description:
-            fields['description'] = description
-        if not add_volumes_new and not remove_volumes_new:
-            # Only update name or description. Set status to available.
-            fields['status'] = 'available'
+        if allow_empty:
+            if name is not None:
+                fields['name'] = name
+            if description is not None:
+                fields['description'] = description
         else:
-            fields['status'] = 'updating'
+            if name:
+                fields['name'] = name
+            if description:
+                fields['description'] = description
 
-        group.update(fields)
-        group.save()
+        # NOTE(geguileo): We will use the updating status in the CG as a lock
+        # mechanism to prevent volume add/remove races with other API, while we
+        # figure out if we really need to add or remove volumes.
+        if add_volumes or remove_volumes:
+            fields['status'] = c_fields.ConsistencyGroupStatus.UPDATING
+
+            # We cannot modify the members of this CG if the CG is being used
+            # to create another CG or a CGsnapshot is being created
+            filters = [~db.cg_creating_from_src(cg_id=group.id),
+                       ~db.cgsnapshot_creating_from_src()]
+        else:
+            filters = []
+
+        expected = {'status': c_fields.ConsistencyGroupStatus.AVAILABLE}
+        if not group.conditional_update(fields, expected, filters):
+            msg = _("Cannot update consistency group %s, status must be "
+                    "available, and it cannot be the source for an ongoing "
+                    "CG or CG Snapshot creation.") % group.id
+            raise exception.InvalidConsistencyGroup(reason=msg)
+
+        # Now the CG is "locked" for updating
+        try:
+            # Validate volumes in add_volumes and remove_volumes.
+            add_volumes_new = self._validate_add_volumes(
+                context, group.volumes, add_volumes_list, group)
+            remove_volumes_new = self._validate_remove_volumes(
+                group.volumes, remove_volumes_list, group)
+
+            self._check_update(group, name, description, add_volumes_new,
+                               remove_volumes_new, allow_empty)
+        except Exception:
+            # If we have an error on the volume_lists we must return status to
+            # available as we were doing before removing API races
+            with excutils.save_and_reraise_exception():
+                group.status = c_fields.ConsistencyGroupStatus.AVAILABLE
+                group.save()
 
         # Do an RPC call only if the update request includes
         # adding/removing volumes. add_volumes_new and remove_volumes_new
@@ -573,9 +563,16 @@ class API(base.Base):
                 context, group,
                 add_volumes=add_volumes_new,
                 remove_volumes=remove_volumes_new)
+        # If there are no new volumes to add or remove and we had changed
+        # the status to updating, turn it back to available
+        elif group.status == c_fields.ConsistencyGroupStatus.UPDATING:
+            group.status = c_fields.ConsistencyGroupStatus.AVAILABLE
+            group.save()
 
     def _validate_remove_volumes(self, volumes, remove_volumes_list, group):
         # Validate volumes in remove_volumes.
+        if not remove_volumes_list:
+            return None
         remove_volumes_new = ""
         for volume in volumes:
             if volume['id'] in remove_volumes_list:
@@ -606,6 +603,8 @@ class API(base.Base):
         return remove_volumes_new
 
     def _validate_add_volumes(self, context, volumes, add_volumes_list, group):
+        if not add_volumes_list:
+            return None
         add_volumes_new = ""
         for volume in volumes:
             if volume['id'] in add_volumes_list:
@@ -715,19 +714,6 @@ class API(base.Base):
         return groups
 
     def create_cgsnapshot(self, context, group, name, description):
-        return self._create_cgsnapshot(context, group, name, description)
-
-    def _create_cgsnapshot(self, context,
-                           group, name, description):
-        volumes = self.db.volume_get_all_by_group(
-            context.elevated(),
-            group.id)
-
-        if not volumes:
-            msg = _("Consistency group is empty. No cgsnapshot "
-                    "will be created.")
-            raise exception.InvalidConsistencyGroup(reason=msg)
-
         options = {'consistencygroup_id': group.id,
                    'user_id': context.user_id,
                    'project_id': context.project_id,
@@ -744,13 +730,16 @@ class API(base.Base):
 
             snap_name = cgsnapshot.name
             snap_desc = cgsnapshot.description
-            self.volume_api.create_snapshots_in_db(
-                context, volumes, snap_name, snap_desc, True, cgsnapshot_id)
+            with group.obj_as_admin():
+                self.volume_api.create_snapshots_in_db(
+                    context, group.volumes, snap_name, snap_desc,
+                    cgsnapshot_id)
 
         except Exception:
             with excutils.save_and_reraise_exception():
                 try:
-                    if cgsnapshot:
+                    # If the cgsnapshot has been created
+                    if cgsnapshot.obj_attr_is_set('id'):
                         cgsnapshot.destroy()
                 finally:
                     LOG.error(_LE("Error occurred when creating cgsnapshot"
@@ -761,11 +750,15 @@ class API(base.Base):
         return cgsnapshot
 
     def delete_cgsnapshot(self, context, cgsnapshot, force=False):
-        if cgsnapshot.status not in ["available", "error"]:
-            msg = _("Cgsnapshot status must be available or error")
+        values = {'status': 'deleting'}
+        expected = {'status': ('available', 'error')}
+        filters = [~db.cg_creating_from_src(cgsnapshot_id=cgsnapshot.id)]
+        res = cgsnapshot.conditional_update(values, expected, filters)
+
+        if not res:
+            msg = _('CgSnapshot status must be available or error, and no CG '
+                    'can be currently using it as source for its creation.')
             raise exception.InvalidCgSnapshot(reason=msg)
-        cgsnapshot.update({'status': 'deleting'})
-        cgsnapshot.save()
         self.volume_rpcapi.delete_cgsnapshot(context.elevated(), cgsnapshot)
 
     def update_cgsnapshot(self, context, cgsnapshot, fields):
