@@ -17,9 +17,6 @@
 Tests dealing with HTTP rate-limiting.
 """
 
-from xml.dom import minidom
-
-from lxml import etree
 from oslo_serialization import jsonutils
 import six
 from six.moves import http_client
@@ -28,9 +25,9 @@ import webob
 
 from cinder.api.v1 import limits
 from cinder.api import views
-from cinder.api import xmlutil
 import cinder.context
 from cinder import test
+from cinder.tests.unit import fake_constants as fake
 
 
 TEST_LIMITS = [
@@ -82,7 +79,7 @@ class LimitsControllerTest(BaseLimitTestSuite):
             "action": "index",
             "controller": "",
         })
-        context = cinder.context.RequestContext('testuser', 'testproject')
+        context = cinder.context.RequestContext(fake.USER_ID, fake.PROJECT_ID)
         request.environ["cinder.context"] = context
         return request
 
@@ -248,7 +245,7 @@ class LimitMiddlewareTest(BaseLimitTestSuite):
 
     def test_limit_class(self):
         """Test that middleware selected correct limiter class."""
-        assert isinstance(self.app._limiter, TestLimiter)
+        self.assertIsInstance(self.app._limiter, TestLimiter)
 
     def test_good_request(self):
         """Test successful GET request through middleware."""
@@ -275,26 +272,6 @@ class LimitMiddlewareTest(BaseLimitTestSuite):
         value = body["overLimitFault"]["details"].strip()
         self.assertEqual(expected, value)
 
-    def test_limited_request_xml(self):
-        """Test a rate-limited (413) response as XML."""
-        request = webob.Request.blank("/")
-        response = request.get_response(self.app)
-        self.assertEqual(200, response.status_int)
-
-        request = webob.Request.blank("/")
-        request.accept = "application/xml"
-        response = request.get_response(self.app)
-        self.assertEqual(413, response.status_int)
-
-        root = minidom.parseString(response.body).childNodes[0]
-        expected = "Only 1 GET request(s) can be made to * every minute."
-
-        details = root.getElementsByTagName("details")
-        self.assertEqual(1, details.length)
-
-        value = details.item(0).firstChild.data.strip()
-        self.assertEqual(expected, value)
-
 
 class LimitTest(BaseLimitTestSuite):
     """Tests for the `limits.Limit` class."""
@@ -362,7 +339,7 @@ class ParseLimitsTest(BaseLimitTestSuite):
                                             '(POST, /bar*, /bar.*, 5, second);'
                                             '(Say, /derp*, /derp.*, 1, day)')
         except ValueError as e:
-            assert False, e
+            self.fail(msg=e)
 
         # Make sure the number of returned limits are correct
         self.assertEqual(4, len(l))
@@ -819,89 +796,3 @@ class LimitsViewBuilderTest(test.TestCase):
         rate_limits = []
         output = self.view_builder.build(rate_limits, abs_limits)
         self.assertDictMatch(expected_limits, output)
-
-
-class LimitsXMLSerializationTest(test.TestCase):
-    def test_xml_declaration(self):
-        serializer = limits.LimitsTemplate()
-
-        fixture = {"limits": {
-                   "rate": [],
-                   "absolute": {}}}
-
-        output = serializer.serialize(fixture)
-        has_dec = output.startswith(b"<?xml version='1.0' encoding='UTF-8'?>")
-        self.assertTrue(has_dec)
-
-    def test_index(self):
-        serializer = limits.LimitsTemplate()
-        fixture = {
-            "limits": {
-                "rate": [{
-                    "uri": "*",
-                    "regex": ".*",
-                    "limit": [{
-                        "value": 10,
-                        "verb": "POST",
-                        "remaining": 2,
-                        "unit": "MINUTE",
-                        "next-available": "2011-12-15T22:42:45Z"}]},
-                    {"uri": "*/servers",
-                     "regex": "^/servers",
-                     "limit": [{
-                         "value": 50,
-                         "verb": "POST",
-                         "remaining": 10,
-                         "unit": "DAY",
-                         "next-available": "2011-12-15T22:42:45Z"}]}],
-                "absolute": {"maxServerMeta": 1,
-                             "maxImageMeta": 1,
-                             "maxPersonality": 5,
-                             "maxPersonalitySize": 10240}}}
-
-        output = serializer.serialize(fixture)
-        root = etree.XML(output)
-        xmlutil.validate_schema(root, 'limits')
-
-        # verify absolute limits
-        absolutes = root.xpath('ns:absolute/ns:limit', namespaces=NS)
-        self.assertEqual(4, len(absolutes))
-        for limit in absolutes:
-            name = limit.get('name')
-            value = limit.get('value')
-            self.assertEqual(str(fixture['limits']['absolute'][name]), value)
-
-        # verify rate limits
-        rates = root.xpath('ns:rates/ns:rate', namespaces=NS)
-        self.assertEqual(2, len(rates))
-        for i, rate in enumerate(rates):
-            for key in ['uri', 'regex']:
-                self.assertEqual(str(fixture['limits']['rate'][i][key]),
-                                 rate.get(key))
-            rate_limits = rate.xpath('ns:limit', namespaces=NS)
-            self.assertEqual(1, len(rate_limits))
-            for j, limit in enumerate(rate_limits):
-                for key in ['verb', 'value', 'remaining', 'unit',
-                            'next-available']:
-                    self.assertEqual(
-                        str(fixture['limits']['rate'][i]['limit'][j][key]),
-                        limit.get(key))
-
-    def test_index_no_limits(self):
-        serializer = limits.LimitsTemplate()
-
-        fixture = {"limits": {
-                   "rate": [],
-                   "absolute": {}}}
-
-        output = serializer.serialize(fixture)
-        root = etree.XML(output)
-        xmlutil.validate_schema(root, 'limits')
-
-        # verify absolute limits
-        absolutes = root.xpath('ns:absolute/ns:limit', namespaces=NS)
-        self.assertEqual(0, len(absolutes))
-
-        # verify rate limits
-        rates = root.xpath('ns:rates/ns:rate', namespaces=NS)
-        self.assertEqual(0, len(rates))
