@@ -23,10 +23,9 @@ CONF = cfg.CONF
 
 CONF.import_opt('policy_file', 'cinder.policy', group='oslo_policy')
 CONF.import_opt('volume_driver', 'cinder.volume.manager')
-CONF.import_opt('xiv_ds8k_proxy',
-                'cinder.volume.drivers.ibm.xiv_ds8k')
 CONF.import_opt('backup_driver', 'cinder.backup.manager')
-CONF.import_opt('fixed_key', 'cinder.keymgr.conf_key_mgr', group='keymgr')
+CONF.import_opt('api_class', 'cinder.keymgr', group='key_manager')
+CONF.import_opt('fixed_key', 'cinder.keymgr.conf_key_mgr', group='key_manager')
 CONF.import_opt('scheduler_driver', 'cinder.scheduler.manager')
 
 def_vol_type = 'fake_vol_type'
@@ -35,21 +34,23 @@ def_vol_type = 'fake_vol_type'
 def set_defaults(conf):
     conf.set_default('default_volume_type', def_vol_type)
     conf.set_default('volume_driver',
-                     'cinder.tests.unit.fake_driver.FakeISCSIDriver')
+                     'cinder.tests.fake_driver.FakeISCSIDriver')
     conf.set_default('iscsi_helper', 'fake')
     conf.set_default('rpc_backend', 'cinder.openstack.common.rpc.impl_fake')
     conf.set_default('connection', 'sqlite://', group='database')
     conf.set_default('sqlite_synchronous', False, group='database')
     conf.set_default('policy_file', 'cinder.tests.unit/policy.json',
                      group='oslo_policy')
-    conf.set_default(
-        'xiv_ds8k_proxy',
-        'cinder.tests.unit.test_ibm_xiv_ds8k.XIVDS8KFakeProxyDriver')
     conf.set_default('backup_driver', 'cinder.tests.unit.backup.fake_service')
-    conf.set_default('fixed_key', default='0' * 64, group='keymgr')
+    conf.set_default('api_class',
+                     'cinder.keymgr.conf_key_mgr.ConfKeyManager',
+                     group='key_manager')
+    conf.set_default('fixed_key', default='0' * 64, group='key_manager')
     conf.set_default('scheduler_driver',
                      'cinder.scheduler.filter_scheduler.FilterScheduler')
     conf.set_default('state_path', os.path.abspath(
         os.path.join(os.path.dirname(__file__), '..', '..', '..')))
     conf.set_default('policy_dirs', [], group='oslo_policy')
+    # This is where we don't authenticate
     conf.set_default('auth_strategy', 'noauth')
+    conf.set_default('auth_uri', 'fake', 'keystone_authtoken')
