@@ -20,9 +20,9 @@ from cinder.tests.unit import fake_constants as fake
 
 def fake_db_volume(**updates):
     db_volume = {
-        'id': fake.volume_id,
+        'id': fake.VOLUME_ID,
         'size': 1,
-        'name': 'volume-%s' % fake.volume_id,
+        'name': 'volume-%s' % fake.VOLUME_ID,
         'availability_zone': 'fake_availability_zone',
         'status': 'available',
         'attach_status': 'detached',
@@ -52,7 +52,7 @@ def fake_db_volume(**updates):
 
 def fake_db_volume_type(**updates):
     db_volume_type = {
-        'id': fake.volume_type_id,
+        'id': fake.VOLUME_TYPE_ID,
         'name': 'type-1',
         'description': 'A fake volume type',
         'is_public': True,
@@ -78,8 +78,8 @@ def fake_db_volume_type(**updates):
 
 def fake_db_volume_attachment(**updates):
     db_volume_attachment = {
-        'id': fake.attachment_id,
-        'volume_id': fake.volume_id,
+        'id': fake.ATTACHMENT_ID,
+        'volume_id': fake.VOLUME_ID,
     }
 
     for name, field in objects.VolumeAttachment.fields.items():
