@@ -13,6 +13,8 @@
 #    under the License.
 
 import mock
+from oslo_utils import timeutils
+import pytz
 import six
 
 from cinder.db.sqlalchemy import models
@@ -26,14 +28,14 @@ from cinder.tests.unit import utils
 
 
 fake_backup = {
-    'id': fake.backup_id,
-    'volume_id': fake.volume_id,
+    'id': fake.BACKUP_ID,
+    'volume_id': fake.VOLUME_ID,
     'status': fields.BackupStatus.CREATING,
     'size': 1,
     'display_name': 'fake_name',
     'display_description': 'fake_description',
-    'user_id': fake.user_id,
-    'project_id': fake.project_id,
+    'user_id': fake.USER_ID,
+    'project_id': fake.PROJECT_ID,
     'temp_volume_id': None,
     'temp_snapshot_id': None,
     'snapshot_id': None,
@@ -46,10 +48,10 @@ class TestBackup(test_objects.BaseObjectsTestCase):
 
     @mock.patch('cinder.db.get_by_id', return_value=fake_backup)
     def test_get_by_id(self, backup_get):
-        backup = objects.Backup.get_by_id(self.context, fake.user_id)
+        backup = objects.Backup.get_by_id(self.context, fake.USER_ID)
         self._compare(self, fake_backup, backup)
         backup_get.assert_called_once_with(self.context, models.Backup,
-                                           fake.user_id)
+                                           fake.USER_ID)
 
     @mock.patch('cinder.db.sqlalchemy.api.model_query')
     def test_get_by_id_no_existing_id(self, model_query):
@@ -77,13 +79,22 @@ class TestBackup(test_objects.BaseObjectsTestCase):
         backup_update.assert_called_once_with(self.context, backup.id,
                                               {'display_name': 'foobar'})
 
-    @mock.patch('cinder.db.backup_destroy')
-    def test_destroy(self, backup_destroy):
-        backup = objects.Backup(context=self.context, id=fake.backup_id)
+    @mock.patch('oslo_utils.timeutils.utcnow', return_value=timeutils.utcnow())
+    @mock.patch('cinder.db.sqlalchemy.api.backup_destroy')
+    def test_destroy(self, backup_destroy, utcnow_mock):
+        backup_destroy.return_value = {
+            'status': fields.BackupStatus.DELETED,
+            'deleted': True,
+            'deleted_at': utcnow_mock.return_value}
+        backup = objects.Backup(context=self.context, id=fake.BACKUP_ID)
         backup.destroy()
         self.assertTrue(backup_destroy.called)
         admin_context = backup_destroy.call_args[0][0]
         self.assertTrue(admin_context.is_admin)
+        self.assertTrue(backup.deleted)
+        self.assertEqual(fields.BackupStatus.DELETED, backup.status)
+        self.assertEqual(utcnow_mock.return_value.replace(tzinfo=pytz.UTC),
+                         backup.deleted_at)
 
     def test_obj_field_temp_volume_snapshot_id(self):
         backup = objects.Backup(context=self.context,
@@ -104,7 +115,7 @@ class TestBackup(test_objects.BaseObjectsTestCase):
 
     def test_import_record(self):
         utils.replace_obj_loader(self, objects.Backup)
-        backup = objects.Backup(context=self.context, id=fake.backup_id,
+        backup = objects.Backup(context=self.context, id=fake.BACKUP_ID,
                                 parent_id=None,
                                 num_dependent_backups=0)
         export_string = backup.encode_record()
@@ -115,7 +126,7 @@ class TestBackup(test_objects.BaseObjectsTestCase):
 
     def test_import_record_additional_info(self):
         utils.replace_obj_loader(self, objects.Backup)
-        backup = objects.Backup(context=self.context, id=fake.backup_id,
+        backup = objects.Backup(context=self.context, id=fake.BACKUP_ID,
                                 parent_id=None,
                                 num_dependent_backups=0)
         extra_info = {'driver': {'key1': 'value1', 'key2': 'value2'}}
@@ -139,7 +150,7 @@ class TestBackup(test_objects.BaseObjectsTestCase):
 
     def test_import_record_additional_info_cant_overwrite(self):
         utils.replace_obj_loader(self, objects.Backup)
-        backup = objects.Backup(context=self.context, id=fake.backup_id,
+        backup = objects.Backup(context=self.context, id=fake.BACKUP_ID,
                                 parent_id=None,
                                 num_dependent_backups=0)
         export_string = backup.encode_record(id='fake_id')
@@ -169,7 +180,7 @@ class TestBackup(test_objects.BaseObjectsTestCase):
         # On the second backup_get, return the backup with an updated
         # display_name
         backup_get.side_effect = [db_backup1, db_backup2]
-        backup = objects.Backup.get_by_id(self.context, fake.backup_id)
+        backup = objects.Backup.get_by_id(self.context, fake.BACKUP_ID)
         self._compare(self, db_backup1, backup)
 
         # display_name was updated, so a backup refresh should have a new value
@@ -180,9 +191,9 @@ class TestBackup(test_objects.BaseObjectsTestCase):
             call_bool = mock.call.__bool__()
         else:
             call_bool = mock.call.__nonzero__()
-        backup_get.assert_has_calls([mock.call(self.context, fake.backup_id),
+        backup_get.assert_has_calls([mock.call(self.context, fake.BACKUP_ID),
                                      call_bool,
-                                     mock.call(self.context, fake.backup_id)])
+                                     mock.call(self.context, fake.BACKUP_ID)])
 
 
 class TestBackupList(test_objects.BaseObjectsTestCase):
