@@ -13,9 +13,13 @@
 #    under the License.
 
 import mock
+from oslo_utils import timeutils
+import pytz
 import six
 
+from cinder import exception
 from cinder import objects
+from cinder.tests.unit import fake_cluster
 from cinder.tests.unit import fake_service
 from cinder.tests.unit import objects as test_objects
 
@@ -30,25 +34,25 @@ class TestService(test_objects.BaseObjectsTestCase):
         self._compare(self, db_service, service)
         service_get.assert_called_once_with(self.context, 1)
 
-    @mock.patch('cinder.db.service_get_by_host_and_topic')
-    def test_get_by_host_and_topic(self, service_get_by_host_and_topic):
+    @mock.patch('cinder.db.service_get')
+    def test_get_by_host_and_topic(self, service_get):
         db_service = fake_service.fake_db_service()
-        service_get_by_host_and_topic.return_value = db_service
+        service_get.return_value = db_service
         service = objects.Service.get_by_host_and_topic(
             self.context, 'fake-host', 'fake-topic')
         self._compare(self, db_service, service)
-        service_get_by_host_and_topic.assert_called_once_with(
-            self.context, 'fake-host', 'fake-topic')
+        service_get.assert_called_once_with(
+            self.context, disabled=False, host='fake-host', topic='fake-topic')
 
-    @mock.patch('cinder.db.service_get_by_args')
-    def test_get_by_args(self, service_get_by_args):
+    @mock.patch('cinder.db.service_get')
+    def test_get_by_args(self, service_get):
         db_service = fake_service.fake_db_service()
-        service_get_by_args.return_value = db_service
+        service_get.return_value = db_service
         service = objects.Service.get_by_args(
             self.context, 'fake-host', 'fake-key')
         self._compare(self, db_service, service)
-        service_get_by_args.assert_called_once_with(
-            self.context, 'fake-host', 'fake-key')
+        service_get.assert_called_once_with(
+            self.context, host='fake-host', binary='fake-key')
 
     @mock.patch('cinder.db.service_create')
     def test_create(self, service_create):
@@ -69,14 +73,21 @@ class TestService(test_objects.BaseObjectsTestCase):
         service_update.assert_called_once_with(self.context, service.id,
                                                {'topic': 'foobar'})
 
-    @mock.patch('cinder.db.service_destroy')
-    def test_destroy(self, service_destroy):
+    @mock.patch('oslo_utils.timeutils.utcnow', return_value=timeutils.utcnow())
+    @mock.patch('cinder.db.sqlalchemy.api.service_destroy')
+    def test_destroy(self, service_destroy, utcnow_mock):
+        service_destroy.return_value = {
+            'deleted': True,
+            'deleted_at': utcnow_mock.return_value}
         db_service = fake_service.fake_db_service()
         service = objects.Service._from_db_object(
             self.context, objects.Service(), db_service)
         with mock.patch.object(service._context, 'elevated') as elevated_ctx:
             service.destroy()
             service_destroy.assert_called_once_with(elevated_ctx(), 123)
+        self.assertTrue(service.deleted)
+        self.assertEqual(utcnow_mock.return_value.replace(tzinfo=pytz.UTC),
+                         service.deleted_at)
 
     @mock.patch('cinder.db.sqlalchemy.api.service_get')
     def test_refresh(self, service_get):
@@ -102,38 +113,69 @@ class TestService(test_objects.BaseObjectsTestCase):
                                       call_bool,
                                       mock.call(self.context, 123)])
 
-    @mock.patch('cinder.db.service_get_all_by_binary')
-    def _test_get_minimum_version(self, services_update, expected,
-                                  service_get_all_by_binary):
-        services = [fake_service.fake_db_service(**s) for s in services_update]
-        service_get_all_by_binary.return_value = services
-
-        min_rpc = objects.Service.get_minimum_rpc_version(self.context, 'foo')
-        self.assertEqual(expected[0], min_rpc)
-        min_obj = objects.Service.get_minimum_obj_version(self.context, 'foo')
-        self.assertEqual(expected[1], min_obj)
-        service_get_all_by_binary.assert_has_calls(
-            [mock.call(self.context, 'foo', disabled=None)] * 2)
-
-    @mock.patch('cinder.db.service_get_all_by_binary')
-    def test_get_minimum_version(self, service_get_all_by_binary):
+    @mock.patch('cinder.db.service_get_all')
+    def test_get_minimum_version(self, service_get_all):
         services_update = [
             {'rpc_current_version': '1.0', 'object_current_version': '1.3'},
             {'rpc_current_version': '1.1', 'object_current_version': '1.2'},
             {'rpc_current_version': '2.0', 'object_current_version': '2.5'},
         ]
         expected = ('1.0', '1.2')
-        self._test_get_minimum_version(services_update, expected)
+        services = [fake_service.fake_db_service(**s) for s in services_update]
+        service_get_all.return_value = services
 
-    @mock.patch('cinder.db.service_get_all_by_binary')
-    def test_get_minimum_version_liberty(self, service_get_all_by_binary):
+        min_rpc = objects.Service.get_minimum_rpc_version(self.context, 'foo')
+        self.assertEqual(expected[0], min_rpc)
+        min_obj = objects.Service.get_minimum_obj_version(self.context, 'foo')
+        self.assertEqual(expected[1], min_obj)
+        service_get_all.assert_has_calls(
+            [mock.call(self.context, binary='foo', disabled=None)] * 2)
+
+    @mock.patch('cinder.db.service_get_all')
+    def test_get_minimum_version_liberty(self, service_get_all):
         services_update = [
             {'rpc_current_version': '1.0', 'object_current_version': '1.3'},
             {'rpc_current_version': '1.1', 'object_current_version': None},
             {'rpc_current_version': None, 'object_current_version': '2.5'},
         ]
-        expected = ('liberty', 'liberty')
-        self._test_get_minimum_version(services_update, expected)
+        services = [fake_service.fake_db_service(**s) for s in services_update]
+        service_get_all.return_value = services
+
+        self.assertRaises(exception.ServiceTooOld,
+                          objects.Service.get_minimum_rpc_version,
+                          self.context, 'foo')
+        self.assertRaises(exception.ServiceTooOld,
+                          objects.Service.get_minimum_obj_version,
+                          self.context, 'foo')
+
+    @mock.patch('cinder.db.service_get_all')
+    def test_get_minimum_version_no_binary(self, service_get_all):
+        services_update = [
+            {'rpc_current_version': '1.0', 'object_current_version': '1.3'},
+            {'rpc_current_version': '1.1', 'object_current_version': '1.2'},
+            {'rpc_current_version': '2.0', 'object_current_version': '2.5'},
+        ]
+        services = [fake_service.fake_db_service(**s) for s in services_update]
+        service_get_all.return_value = services
+
+        min_obj = objects.Service.get_minimum_obj_version(self.context)
+        self.assertEqual('1.2', min_obj)
+        service_get_all.assert_called_once_with(self.context, binary=None,
+                                                disabled=None)
+
+    @mock.patch('cinder.db.sqlalchemy.api.cluster_get')
+    def test_lazy_loading_cluster_field(self, cluster_get):
+        cluster_orm = fake_cluster.fake_cluster_orm(name='mycluster')
+        cluster_get.return_value = cluster_orm
+        cluster = objects.Cluster._from_db_object(self.context,
+                                                  objects.Cluster(),
+                                                  cluster_orm)
+
+        service = fake_service.fake_service_obj(self.context,
+                                                cluster_name='mycluster')
+        self.assertEqual(cluster, service.cluster)
+        cluster_get.assert_called_once_with(self.context, None,
+                                            name='mycluster')
 
 
 class TestServiceList(test_objects.BaseObjectsTestCase):
@@ -144,30 +186,30 @@ class TestServiceList(test_objects.BaseObjectsTestCase):
 
         filters = {'host': 'host', 'binary': 'foo', 'disabled': False}
         services = objects.ServiceList.get_all(self.context, filters)
-        service_get_all.assert_called_once_with(self.context, filters)
+        service_get_all.assert_called_once_with(self.context, **filters)
         self.assertEqual(1, len(services))
         TestService._compare(self, db_service, services[0])
 
-    @mock.patch('cinder.db.service_get_all_by_topic')
-    def test_get_all_by_topic(self, service_get_all_by_topic):
+    @mock.patch('cinder.db.service_get_all')
+    def test_get_all_by_topic(self, service_get_all):
         db_service = fake_service.fake_db_service()
-        service_get_all_by_topic.return_value = [db_service]
+        service_get_all.return_value = [db_service]
 
         services = objects.ServiceList.get_all_by_topic(
             self.context, 'foo', 'bar')
-        service_get_all_by_topic.assert_called_once_with(
-            self.context, 'foo', disabled='bar')
+        service_get_all.assert_called_once_with(
+            self.context, topic='foo', disabled='bar')
         self.assertEqual(1, len(services))
         TestService._compare(self, db_service, services[0])
 
-    @mock.patch('cinder.db.service_get_all_by_binary')
-    def test_get_all_by_binary(self, service_get_all_by_binary):
+    @mock.patch('cinder.db.service_get_all')
+    def test_get_all_by_binary(self, service_get_all):
         db_service = fake_service.fake_db_service()
-        service_get_all_by_binary.return_value = [db_service]
+        service_get_all.return_value = [db_service]
 
         services = objects.ServiceList.get_all_by_binary(
             self.context, 'foo', 'bar')
-        service_get_all_by_binary.assert_called_once_with(
-            self.context, 'foo', disabled='bar')
+        service_get_all.assert_called_once_with(
+            self.context, binary='foo', disabled='bar')
         self.assertEqual(1, len(services))
         TestService._compare(self, db_service, services[0])
