@@ -13,11 +13,13 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 import json
-import urllib
+
+from six.moves import urllib
 
 from cinder import context
 from cinder import db
 from cinder import exception
+from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import fake_snapshot
 from cinder.tests.unit import fake_volume
 from cinder.tests.unit.volume.drivers.emc import scaleio
@@ -39,18 +41,20 @@ class TestCreateSnapShot(scaleio.TestScaleIODriver):
         ctx = context.RequestContext('fake', 'fake', auth_token=True)
 
         self.fake_volume = fake_volume.fake_volume_obj(
-            ctx, **{'provider_id': 'fake_pid'})
+            ctx, **{'provider_id': fake.PROVIDER_ID})
 
         self.snapshot = fake_snapshot.fake_snapshot_obj(
             ctx, **{'volume': self.fake_volume})
 
-        self.mock_object(db, 'volume_get', self.return_fake_volume)
+        self.mock_object(db.sqlalchemy.api, 'volume_get',
+                         self.return_fake_volume)
 
-        self.volume_name_2x_enc = urllib.quote(
-            urllib.quote(self.driver._id_to_base64(self.snapshot.volume_id))
+        snap_vol_id = self.snapshot.volume_id
+        self.volume_name_2x_enc = urllib.parse.quote(
+            urllib.parse.quote(self.driver._id_to_base64(snap_vol_id))
         )
-        self.snapshot_name_2x_enc = urllib.quote(
-            urllib.quote(self.driver._id_to_base64(self.snapshot.id))
+        self.snapshot_name_2x_enc = urllib.parse.quote(
+            urllib.parse.quote(self.driver._id_to_base64(self.snapshot.id))
         )
 
         self.snapshot_reply = json.dumps(
