@@ -26,6 +26,7 @@ from cinder import compute
 from cinder import exception
 from cinder.i18n import _, _LI, _LW
 from cinder.image import image_utils
+from cinder import interface
 from cinder import utils
 from cinder.volume.drivers import remotefs as remotefs_drv
 
@@ -57,7 +58,8 @@ CONF = cfg.CONF
 CONF.register_opts(volume_opts)
 
 
-class QuobyteDriver(remotefs_drv.RemoteFSSnapDriver):
+@interface.volumedriver
+class QuobyteDriver(remotefs_drv.RemoteFSSnapDriverDistributed):
     """Cinder driver for Quobyte USP.
 
     Volumes are stored as files on the mounted Quobyte volume. The hypervisor
@@ -84,6 +86,9 @@ class QuobyteDriver(remotefs_drv.RemoteFSSnapDriver):
     volume_backend_name = 'Quobyte'
     VERSION = VERSION
 
+    # ThirdPartySystems wiki page
+    CI_WIKI_NAME = "Quobyte_CI"
+
     def __init__(self, execute=processutils.execute, *args, **kwargs):
         super(QuobyteDriver, self).__init__(*args, **kwargs)
         self.configuration.append_config_values(volume_opts)
@@ -185,20 +190,20 @@ class QuobyteDriver(remotefs_drv.RemoteFSSnapDriver):
         """
 
         LOG.debug("snapshot: %(snap)s, volume: %(vol)s, ",
-                  {'snap': snapshot['id'],
-                   'vol': volume['id'],
+                  {'snap': snapshot.id,
+                   'vol': volume.id,
                    'size': volume_size})
 
-        info_path = self._local_path_volume_info(snapshot['volume'])
+        info_path = self._local_path_volume_info(snapshot.volume)
         snap_info = self._read_info_file(info_path)
-        vol_path = self._local_volume_dir(snapshot['volume'])
-        forward_file = snap_info[snapshot['id']]
+        vol_path = self._local_volume_dir(snapshot.volume)
+        forward_file = snap_info[snapshot.id]
         forward_path = os.path.join(vol_path, forward_file)
 
         # Find the file which backs this file, which represents the point
         # when this snapshot was created.
         img_info = self._qemu_img_info(forward_path,
-                                       snapshot['volume']['name'])
+                                       snapshot.volume.name)
         path_to_snap_img = os.path.join(vol_path, img_info.backing_file)
 
         path_to_new_vol = self._local_path_volume(volume)
@@ -221,12 +226,12 @@ class QuobyteDriver(remotefs_drv.RemoteFSSnapDriver):
     def delete_volume(self, volume):
         """Deletes a logical volume."""
 
-        if not volume['provider_location']:
+        if not volume.provider_location:
             LOG.warning(_LW('Volume %s does not have provider_location '
-                            'specified, skipping'), volume['name'])
+                            'specified, skipping'), volume.name)
             return
 
-        self._ensure_share_mounted(volume['provider_location'])
+        self._ensure_share_mounted(volume.provider_location)
 
         volume_dir = self._local_volume_dir(volume)
         mounted_path = os.path.join(volume_dir,
@@ -261,16 +266,16 @@ class QuobyteDriver(remotefs_drv.RemoteFSSnapDriver):
         # Find active qcow2 file
         active_file = self.get_active_image_from_info(volume)
         path = '%s/%s/%s' % (self.configuration.quobyte_mount_point_base,
-                             self._get_hash_str(volume['provider_location']),
+                             self._get_hash_str(volume.provider_location),
                              active_file)
 
-        data = {'export': volume['provider_location'],
+        data = {'export': volume.provider_location,
                 'name': active_file}
-        if volume['provider_location'] in self.shares:
-            data['options'] = self.shares[volume['provider_location']]
+        if volume.provider_location in self.shares:
+            data['options'] = self.shares[volume.provider_location]
 
         # Test file for raw vs. qcow2 format
-        info = self._qemu_img_info(path, volume['name'])
+        info = self._qemu_img_info(path, volume.name)
         data['format'] = info.file_format
         if data['format'] not in ['raw', 'qcow2']:
             msg = _('%s must be a valid raw or qcow2 image.') % path
@@ -299,7 +304,7 @@ class QuobyteDriver(remotefs_drv.RemoteFSSnapDriver):
                     ' driver when no snapshots exist.')
             raise exception.InvalidVolume(msg)
 
-        info = self._qemu_img_info(volume_path, volume['name'])
+        info = self._qemu_img_info(volume_path, volume.name)
         backing_fmt = info.file_format
 
         if backing_fmt not in ['raw', 'qcow2']:
@@ -315,7 +320,7 @@ class QuobyteDriver(remotefs_drv.RemoteFSSnapDriver):
         :param volume: volume reference
         """
         volume_path = self.local_path(volume)
-        volume_size = volume['size']
+        volume_size = volume.size
 
         if self.configuration.quobyte_qcow2_volumes:
             self._create_qcow2_file(volume_path, volume_size)
@@ -330,7 +335,7 @@ class QuobyteDriver(remotefs_drv.RemoteFSSnapDriver):
     def _load_shares_config(self, share_file=None):
         """Put 'quobyte_volume_url' into the 'shares' list.
 
-        :param share_file: string, Not used because the user has to specify the
+        :param share_file: string, Not used because the user has to specify
                                    the Quobyte volume directly.
         """
         self.shares = {}
