@@ -33,6 +33,8 @@ from cinder import quota
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 QUOTAS = quota.QUOTAS
+ENCRYPTION_IGNORED_FIELDS = ['volume_type_id', 'created_at', 'updated_at',
+                             'deleted_at']
 
 
 def create(context,
@@ -90,9 +92,8 @@ def destroy(context, id):
     if id is None:
         msg = _("id cannot be None")
         raise exception.InvalidVolumeType(reason=msg)
-    else:
-        elevated = context if context.is_admin else context.elevated()
-        db.volume_type_destroy(elevated, id)
+    elevated = context if context.is_admin else context.elevated()
+    return db.volume_type_destroy(elevated, id)
 
 
 def get_all_types(context, inactive=0, filters=None, marker=None,
@@ -111,6 +112,12 @@ def get_all_types(context, inactive=0, filters=None, marker=None,
     return vol_types
 
 
+def get_all_types_by_group(context, group_id):
+    """Get all volume_types in a group."""
+    vol_types = db.volume_type_get_all_by_group(context, group_id)
+    return vol_types
+
+
 def get_volume_type(ctxt, id, expected_fields=None):
     """Retrieves single volume type by id."""
     if id is None:
@@ -209,6 +216,7 @@ def get_volume_type_encryption(context, volume_type_id):
 
 
 def get_volume_type_qos_specs(volume_type_id):
+    """Get all qos specs for given volume type."""
     ctxt = context.get_admin_context()
     res = db.volume_type_qos_specs_get(ctxt,
                                        volume_type_id)
@@ -221,15 +229,26 @@ def volume_types_diff(context, vol_type_id1, vol_type_id2):
     Returns a tuple of (diff, equal), where 'equal' is a boolean indicating
     whether there is any difference, and 'diff' is a dictionary with the
     following format:
-    {'extra_specs': {'key1': (value_in_1st_vol_type, value_in_2nd_vol_type),
-                     'key2': (value_in_1st_vol_type, value_in_2nd_vol_type),
-                     ...}
-     'qos_specs': {'key1': (value_in_1st_vol_type, value_in_2nd_vol_type),
-                   'key2': (value_in_1st_vol_type, value_in_2nd_vol_type),
-                   ...}
-     'encryption': {'cipher': (value_in_1st_vol_type, value_in_2nd_vol_type),
-                   {'key_size': (value_in_1st_vol_type, value_in_2nd_vol_type),
-                    ...}
+
+    .. code-block:: json
+
+        {
+            'extra_specs': {'key1': (value_in_1st_vol_type,
+                                     value_in_2nd_vol_type),
+                            'key2': (value_in_1st_vol_type,
+                                     value_in_2nd_vol_type),
+                            {...}}
+            'qos_specs': {'key1': (value_in_1st_vol_type,
+                                   value_in_2nd_vol_type),
+                          'key2': (value_in_1st_vol_type,
+                                   value_in_2nd_vol_type),
+                          {...}}
+            'encryption': {'cipher': (value_in_1st_vol_type,
+                                      value_in_2nd_vol_type),
+                          {'key_size': (value_in_1st_vol_type,
+                                        value_in_2nd_vol_type),
+                           {...}}
+        }
     """
     def _fix_qos_specs(qos_specs):
         if qos_specs:
@@ -240,8 +259,7 @@ def volume_types_diff(context, vol_type_id1, vol_type_id2):
     def _fix_encryption_specs(encryption):
         if encryption:
             encryption = dict(encryption)
-            for param in ['volume_type_id', 'created_at', 'updated_at',
-                          'deleted_at']:
+            for param in ENCRYPTION_IGNORED_FIELDS:
                 encryption.pop(param, None)
         return encryption
 
@@ -296,3 +314,19 @@ def volume_types_diff(context, vol_type_id1, vol_type_id2):
         all_equal = False
 
     return (diff, all_equal)
+
+
+def volume_types_encryption_changed(context, vol_type_id1, vol_type_id2):
+    """Return whether encryptions of two volume types are same."""
+    def _get_encryption(enc):
+        enc = dict(enc)
+        for param in ENCRYPTION_IGNORED_FIELDS:
+            enc.pop(param, None)
+        return enc
+
+    enc1 = get_volume_type_encryption(context, vol_type_id1)
+    enc2 = get_volume_type_encryption(context, vol_type_id2)
+
+    enc1_filtered = _get_encryption(enc1) if enc1 else None
+    enc2_filtered = _get_encryption(enc2) if enc2 else None
+    return enc1_filtered != enc2_filtered
